#!/bin/sh
# based on idea from https://unix.stackexchange.com/a/436713
# executes in parallel the code between #beginpar and #endpar, e.g.:
# NB: output will be mangled, i.e. shared among all parallel processes
# NB: be sure that parallel code writes to different files

if [ $# != 1 ]; then
    echo 'Executes a script in parallel'
    echo "Usage: PARALLEL=4 `basename $0` script.sh"
    echo 'where script.sh is for example:'
    echo '  for i in `seq 8`; do'
    echo '    #beginpar'
    echo '    echo $i start; sleep $i; echo $i stop'
    echo '    #endpar'
    echo '  done'
    echo 'Use #waitpar if you need to wait all the results and process them afterwards'
    exit 1
fi

N=${PARALLEL:-3}
N=`expr $N + 1`
parfile=$1.par.sh

cp -a $1 $parfile
sed -i '1s|^|#!/bin/bash\n|' $parfile
sed -i 's/#beginpar/(/g' $parfile
# allow to execute max $N jobs in parallel; wait only for the first job
sed -i 's/#endpar/)\&\n[[ $(jobs -r -p | wc -l) -gt '$N' ]] \&\& wait -n/g' $parfile
# add wait either at #waitpar, or at the end of file
if grep -q \#waitpar $parfile; then
    sed -i 's/#waitpar/wait/g' $parfile
else
    /bin/echo -e '\nwait' >>$parfile
fi

./$parfile
rm $parfile
